-- $Id: DropDown.lua 43 2014-02-23 18:40:01Z diesal@reece-tech.com $

local DiesalGUI = LibStub('DiesalGUI-1.0')
-- ~~| Libraries |~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
local DiesalTools = LibStub('DiesalTools-1.0')
-- ~~| Lua Upvalues |~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
local tsort																	= table.sort
local sub, format, lower, upper,gsub								= string.sub, string.format, string.lower, string.upper, string.gsub 

-- ~~| WoW Upvalues |~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

-- ~~| Dropdown |~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
local TYPE 		= 'Dropdown'
local VERSION 	= 2
-- ~~| Dropdown StyleSheets |~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
local styleSheet = {	
	['frame-background'] = {			
		type			= 'texture',
		layer			= 'BACKGROUND',
		gradient		= 'VERTICAL',							
		color			= '000000',		
		offset		= 0,		
	},
	['frame-highlight'] = {			
		type			= 'texture',
		layer			= 'BORDER',
		gradient		= 'VERTICAL',							
		color			= 'ffffff',			
		alpha 		= .20,
		alphaEnd		= .30,
		offset		= -1,		
	},
	['frame-inline'] = {		
		type			= 'outline',
		layer			= 'BORDER',
		gradient		= 'VERTICAL',	
		color			= 'ffffff',
		alpha 		= .03,
		alphaEnd		= .07,
		offset		= -1,	
	},		
	['frame-hover'] = {		
		type			= 'outline',
		layer			= 'HIGHLIGHT',	
		color			= '00a8ff',
		alpha			= .3,
		offset		= -1,		
	},
	['frame-arrow'] = {			
		type			= 'texture',
		layer			= 'BORDER',								
		texFile		= 'DiesalGUIcons',
		texCoord		= {2,1,16,256,128},				
		alpha 		= .7,		
		offset		= {nil,-4,-6,nil},
		height		= 5,
		width			= 6,				
	},	
	['text-color'] = {
		type			= 'Font',
		color			= 'cccccc',
	},
	['dropdown-background'] = {				
		type			= 'texture',
		layer			= 'BACKGROUND',
		color			= '000000',			
	},	
	['dropdown-inline'] = {	
		type			= 'outline',
		layer			= 'BORDER',
		color			= 'ffffff',		
		alpha		 	= .1,
		offset		= -1,				
	},
	['dropdown-highlight'] = {
		type			= 'texture',
		layer			= 'BORDER',
		color			= 'ffffff',
		gradient		= 'VERTICAL',
		alpha 		= .02,
		alphaEnd		= .07,
		offset		= -1,
		height		= 10,
	},
	['dropdown-shadow'] = {			
		type			= 'shadow',		
	},	
}	
-- ~~| Dropdown Locals |~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
local function compare(a,b)	
	return  a.value < b.value	
end
local function sortList(list,orderedKeys)
	if orderedKeys then return orderedKeys end
	local sortedList 	= {}
	local orderedKeys = {}
	for key,value in pairs(list) do
		sortedList[#sortedList + 1] = {key=key,value=value}
	end
	tsort(sortedList,compare)
	for i,value in ipairs(sortedList) do
		orderedKeys[#orderedKeys + 1] = value.key
	end
	
	return orderedKeys
end
-- ~~| Dropdown Methods |~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
local methods = {
	['OnAcquire'] = function(self)
		self:ApplySettings()
		self:AddStyleSheet(styleSheet)
		-- self:AddStyleSheet(wireFrameSheet)
		self:Show()
	end,
	['OnRelease'] = function(self)
		
	end,
	['ApplySettings'] = function(self)
		local settings 	= self.settings
		local content		= self.content
		local frame			= self.frame
		local children 	= self.children
		local dropdown		= self.dropdown

		frame:SetWidth(settings.width)
		frame:SetHeight(settings.height)
		
		content:SetPoint("TOPLEFT",settings.dropdownPadLeft,-settings.dropdownPadTop)
		content:SetPoint("BOTTOMRIGHT",-settings.dropdownPadRight,settings.dropdownPadBottom)
			
		local menuHeight	= 0			
		for i=1 , #children do 
			menuHeight = menuHeight + children[i].frame:GetHeight()				
		end 
		dropdown:SetHeight(settings.dropdownPadTop + menuHeight + settings.dropdownPadBottom)			
		
	end,	
	['SetList'] = function(self,list,orderedKeys)
		self:ReleaseChildren()
		local settings		= self.settings
		local orderedKeys = sortList(list,orderedKeys) 		
		settings.list 		= list		
		
		for position, key in ipairs(orderedKeys) do				 				
			local dropdownItem = DiesalGUI:Create('DropdownItem')
			self:AddChild(dropdownItem)
			dropdownItem:SetParentObject(self)
			dropdownItem:SetSettings({
				key		= key,
				value		= list[key],
				position	= position,
			},true)			
		end
		self:ApplySettings()		
	end,
	['SetValue'] = function(self,key)			
		local dropdownItems 	= self.children
		local selectionTable = {}
		local selectedKey
		local dropdownText		
		
		if self.settings.multiSelect then					
			for i=1,#dropdownItems do										
				if dropdownItems[i].settings.key == key then	selectedKey = key; dropdownItems[i]:SetSelected(true) end				
				if dropdownItems[i].settings.selected then
					if dropdownText then
						dropdownText = format('%s, %s',dropdownText,dropdownItems[i].settings.value)						
					else 
						dropdownText = dropdownItems[i].settings.value
					end
					selectionTable[#selectionTable+1] = dropdownItems[i].settings.key
				end		
			end		
		else
			for i=1,#dropdownItems do
				dropdownItems[i]:SetSelected(false)			
				if dropdownItems[i].settings.key == key then
					dropdownItems[i]:SetSelected(true)
					dropdownText 	= dropdownItems[i].settings.value
					selectionTable	= {key}
					selectedKey 	= key				
				end
			end
		end
		if selectedKey then 
			self:SetText(dropdownText)
			self:FireEvent("OnValueChanged",selectedKey,true,selectionTable)	
		end					
	end,
	['SetValueTable'] = function(self,keyTable)
		if not self.settings.multiSelect or type(keyTable) ~='table' then return end
		local dropdownItems 	= self.children
		local selectionTable = {}
		local selectedKey
		local dropdownText		
		
		for i=1,#dropdownItems do
			local dropdownItem = dropdownItems[i]
			dropdownItem:SetSelected(false)
			for _,key in ipairs(keyTable) do	
						
				if dropdownItem.settings.key == key then
					dropdownItem:SetSelected(true)
					if dropdownText then
						dropdownText = format('%s, %s',dropdownText,dropdownItem.settings.value)						
					else 
						dropdownText = dropdownItem.settings.value
					end
					selectionTable[#selectionTable+1] = dropdownItem.settings.key					
				end
			end			
		end
		self:FireEvent("OnValueChanged",nil,nil,selectionTable)			
		self:SetText(dropdownText)
	end, 		
	['SetMultiSelect'] = function(self,state)
		local settings = self.settings
		settings.multiSelect = state
		
	end,
	['SetText'] = function(self,text)
		self.text:SetText(text)	
	end,
	['SetFocus'] = function(self)
		DiesalGUI:SetFocus(self)		
	end,
	['ClearFocus'] = function(self)
		self.dropdown:Hide()	
	end,
	['EnableMouse'] = function(self,state)
		self.frame:EnableMouse(state)	
	end,
}
-- ~~| Dropdown Constructor |~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
local function Constructor()
	local self 		= DiesalGUI:GetObjectBase(TYPE,'Button')
	local frame 	= self.frame
	-- ~~ Default Settings ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	self.defaults = {
		dropdownPadLeft	= 4,
		dropdownPadRight	= 4,
		dropdownPadTop		= 4,
		dropdownPadBottom	= 4,
		itemHeight			= 16,		
		width					= 100,
		height				= 16,
	}
	-- ~~ Registered Events ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	-- OnValueChanged, OnValueSelected, OnEnter, OnLeave
	-- ~~ Construct ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	frame:SetScript("OnMouseUp", function(this, button)
		if button =='LeftButton' then		
			local dropdown = self.dropdown			
			local visible = dropdown:IsVisible()
			DiesalGUI:OnMouse(this,button)	
			dropdown[visible and "Hide" or 'Show'](dropdown)		
		end
		self:FireEvent("OnMouseUp", button)
	end)
	frame:SetScript("OnEnter",function(this,...)		
		self:FireEvent("OnEnter",...)				
	end)
	frame:SetScript("OnLeave", function(this,...)		
		self:FireEvent("OnLeave",...)					
	end)		
	
	local text = self:CreateRegion("FontString", 'text', frame)	
	text:ClearAllPoints()
	text:SetTextColor(.8,.8,.8,1)		
	text:SetPoint("TOPLEFT", 4, -1)
	text:SetPoint("BOTTOMRIGHT", -14, 1)
	text:SetJustifyV("MIDDLE")	
	text:SetJustifyH("RIGHT")	
	
	local dropdown = self:CreateRegion("Frame", 'dropdown', frame)		
	dropdown:SetFrameStrata("FULLSCREEN_DIALOG")
	dropdown:SetPoint('TOPRIGHT',frame,'BOTTOMRIGHT',0,-2)
	dropdown:SetPoint('TOPLEFT',frame,'BOTTOMLEFT',0,-2)	
	dropdown:SetScript("OnShow", function(this) self:SetFocus() end)
	dropdown:Hide()
	
	self:CreateRegion("Frame", 'content', 	dropdown)	
	-- ~~ Methods ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	for method, func in pairs(methods) do
		self[method] = func
	end
	-- ~~ ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	DiesalGUI:RegisterObject(self)
	return self
end
DiesalGUI:RegisterObjectConstructor(TYPE,Constructor,VERSION)
