-- $Id: Window.lua 43 2014-02-23 18:40:01Z diesal@reece-tech.com $

local DiesalGUI = LibStub("DiesalGUI-1.0")
-- ~~| Libraries |~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
local DiesalTools = LibStub("DiesalTools-1.0")
local DiesalStyle = LibStub("DiesalStyle-1.0")
-- ~~| Lua Upvalues |~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
local type, select, pairs, tonumber									= type, select, pairs, tonumber
local setmetatable, getmetatable, next								= setmetatable, getmetatable, next
local sub, format, lower, upper 										= string.sub, string.format, string.lower, string.upper 
local floor, ceil, abs, modf											= math.floor, math.ceil, math.abs, math.modf	
-- ~~| WoW Upvalues |~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
local CreateFrame, UIParent, GetCursorPosition 					= CreateFrame, UIParent, GetCursorPosition
local GetScreenWidth, GetScreenHeight								= GetScreenWidth, GetScreenHeight
-- ~~| Window |~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
local TYPE 		= "Window"
local VERSION 	= 13
-- ~~| Window StyleSheets |~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
local styleSheet = {
	['frame-background'] = {				
		type			= 'texture',
		layer			= 'BACKGROUND',		
		color			= '0f0f0f',		
		offset		= 0,		
	},
	['frame-shadow'] = {			
		type			= 'shadow',		
	},	
	['frame-outline'] = {				
		type			= 'outline',
		layer			= 'BORDER',
		color			= '000000',
		offset		= 0,	
	},	
	['frame-inline'] = {			
		type			= 'outline',
		layer			= 'BORDER',
		color			= '000000',	
		offset		= {-2,-2,-17,-2},	
	},		
	['titleBar-outline'] = {			
		type			= 'outline',
		layer			= 'ARTWORK',		
		gradient		= 'VERTICAL',				
		color			= 'FFFFFF',		
		alpha			= .0,
		alphaEnd		= .07,		
		offset		= -1,	
	},	
	['titletext-Font'] = {			
		type			= 'font',		
		color			= 'd8d8d8',		
	},	
	['closeButton-icon'] = {			
		type			= 'texture',
		layer			= 'ARTWORK',
		texFile		= 'DiesalGUIcons',
		texCoord		= {9,5,16,256,128},
		alpha 		= .4,	
		offset		= {-2,nil,-2,nil},
		width			= 16,
		height		= 16,			
	},	
	['closeButton-iconHover'] = {		
		type			= 'texture',
		layer			= 'HIGHLIGHT',	
		texFile		= 'DiesalGUIcons',
		texCoord		= {9,5,16,256,128},
		texColor		= 'b30000',
		alpha			= 1,
		offset		= {-2,nil,-2,nil},
		width			= 16,
		height		= 16,			
	},	
	['header-background'] = {			
		type			= 'texture',
		layer			= 'BACKGROUND',
		gradient		= 'VERTICAL',						
		color			= '272727',
		colorEnd		= '232323',
		offset 		= {0,0,0,-1},		
	},	
	['header-outline'] = {			
		type			= 'outline',
		layer			= 'BORDER',
		gradient		= 'VERTICAL',		
		color			= 'FFFFFF',		
		alpha			= .05,
		alphaEnd		= .02,
		offset		= {0,0,0,-1},			
	},	
	['header-divider'] = {			
		type			= 'texture',
		layer			= 'BACKGROUND',
		color			= '040404',
		offset		= {0,0,nil,0},	
		height		= 1,	
	},
	['content-background'] = {		
		type			= 'texture',
		layer			= 'BACKGROUND',
		color			= '121212',
	},
	['content-outline'] = {		
		type			= 'outline',
		layer			= 'BORDER',
		color			= 'FFFFFF',		
		alpha			= .03,
		offset		= 0,
	},		
	['footer-background'] = {			
		type			= 'texture',
		layer			= 'BACKGROUND',
		gradient		= 'VERTICAL',						
		color			= '232323',
		colorEnd		= '272727',
		offset 		= {0,0,-1,0},		
	},	
	['footer-outline'] = {			
		type			= 'outline',
		layer			= 'BORDER',
		color			= 'FFFFFF',
		gradient		= 'VERTICAL',
		alpha			= .02,
		alphaEnd		= .05,
		offset		= {0,0,-1,0},		
	},	
	['footer-divider'] = {			
		type			= 'texture',
		layer			= 'BACKGROUND',		
		color			= '040404',	
		offset		= {0,0,0,nil},	
		height		= 1,	
	},	
}
local wireFrame = {	
	['frame-white'] = {				
		type			= 'outline',
		layer			= 'OVERLAY',
		color			= 'ffffff',	
	},		
	['titleBar-yellow'] = {			
		type			= 'outline',
		layer			= 'OVERLAY',
		color			= 'fffc00',		
	},	
	['closeButton-orange'] = {			
		type			= 'outline',
		layer			= 'OVERLAY',
		color			= 'ffd400',		
	},	
	['header-blue'] = {			
		type			= 'outline',
		layer			= 'OVERLAY',
		color			= '00aaff',			
	},	
	['footer-green'] = {			
		type			= 'outline',
		layer			= 'OVERLAY',
		color			= '55ff00',					
	},	
	
}
local sizerWireFrame = {
	['sizerR-yellow'] = {			
		type			= 'outline',
		layer			= 'OVERLAY',
		color			= 'fffc00',					
	},	
	['sizerB-green'] = {			
		type			= 'outline',
		layer			= 'OVERLAY',
		color			= '55ff00',					
	},	
	['sizerBR-blue'] = {			
		type			= 'outline',
		layer			= 'OVERLAY',
		color			= '00aaff',					
	},	
}
-- ~~| Window Locals |~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-- ~~| Window Methods |~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
local methods = {		
	['OnAcquire'] = function(self)				
		self:ApplySettings()
		self:AddStyleSheet(styleSheet)
		-- self:AddStyleSheet(sizerWireFrame)						
		self:Show()
	end,	
	['OnRelease'] = function(self)				
									
	end,
	['SetTopLevel'] = function(self)				
		DiesalGUI:SetTopLevel(self.frame)							
	end,		
	['ApplySettings'] = function(self)		
		local settings				= self.settings
		local frame 				= self.frame
		local titleBar				= self.titleBar
		local closeButton			= self.closeButton
		local header				= self.header
		local content				= self.content		
		local footer				= self.footer			
		
		local headerHeight 		= settings.header and settings.headerHeight or 0 
		local footerHeight 		= settings.footer and settings.footerHeight or 0 		
		
		self:UpdatePosition()
		self:UpdateSizers()
		
		frame:SetMinResize(settings.minWidth,settings.minHeight)	
		frame:SetMaxResize(settings.maxWidth,settings.maxHeight)	
		titleBar:SetHeight(settings.titleBarHeight)
		closeButton:SetHeight(settings.titleBarHeight)
		closeButton:SetWidth(settings.titleBarHeight)
		
		content:SetPoint("TOPLEFT",settings.padLeft,-(settings.titleBarHeight + headerHeight))
		content:SetPoint("BOTTOMRIGHT",-settings.padRight,footerHeight + settings.padBottom)
		header:SetHeight(headerHeight)
		footer:SetHeight(footerHeight)
		header[settings.header and "Show" or "Hide"](header)
		footer[settings.footer and "Show" or "Hide"](footer)		
		
		header:SetPoint("TOPLEFT",self.titleBar,"BOTTOMLEFT",settings.padLeft,0)
		header:SetPoint("TOPRIGHT",self.titleBar,"BOTTOMRIGHT",-settings.padRight,0)
		
		footer:SetPoint("BOTTOMLEFT",settings.padLeft,settings.padBottom)
		footer:SetPoint("BOTTOMRIGHT",-settings.padRight,settings.padBottom)			
								
	end,
	['UpdatePosition'] = function(self)
		local settings = self.settings
		local frame		= self.frame
		
		frame:SetWidth(settings.width)
		frame:SetHeight(settings.height)
		
		if settings.top and settings.left then
			frame:SetPoint("TOP",UIParent,"BOTTOM",0,settings.top)
			frame:SetPoint("LEFT",UIParent,"LEFT",settings.left,0)
		else			
			frame:SetPoint("CENTER",UIParent,"CENTER")
		end						
	end,	
	['UpdateSizers'] = function(self)
		local settings = self.settings
		
		local frame		= self.frame
		local sizerB	= self.sizerB
		local sizerR	= self.sizerR
		local sizerBR	= self.sizerBR			
		
		sizerBR[settings.sizerBR and "Show" or "Hide"](sizerBR)
		sizerBR:SetSize(settings.sizerBRWidth,settings.sizerBRHeight)		
		sizerB[settings.sizerB and "Show" or "Hide"](sizerB)
		sizerB:SetHeight(settings.sizerHeight)		
		sizerB:SetPoint("BOTTOMRIGHT",frame,"BOTTOMRIGHT",-settings.sizerBRWidth,0)
		sizerR[settings.sizerR and "Show" or "Hide"](sizerR)	
		sizerR:SetWidth(settings.sizerWidth)
		sizerR:SetPoint("TOPRIGHT",frame,"TOPRIGHT",0,-settings.titleBarHeight)			
		sizerR:SetPoint("BOTTOMRIGHT",frame,"BOTTOMRIGHT",0,settings.sizerBRHeight)						
	end,		
	['SetTitle'] = function(self,title,subtitle)
		local settings = self.settings
		settings.title		= title 		or settings.title
		settings.subTitle	= subtitle 	or settings.subTitle		
		self.titletext:SetText(format('%s |cff7f7f7f%s',settings.title,settings.subTitle))					
	end,		
}
-- ~~| Window Constructor |~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
local function Constructor()
	local self 		= DiesalGUI:GetObjectBase(TYPE)
	local frame 	= self.frame		
	-- ~~ Default Settings ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	self.defaults = {
		height 				= 300,
		width 				= 500,
		minHeight 			= 200,
		minWidth 			= 200,
		maxHeight 			= 9999,
		maxWidth 			= 9999,
		left					= nil,
		top					= nil,
		titleBarHeight		= 18,
		title					= '',
		subTitle				= '',	
		padLeft				= 3,
		padRight				= 3,	
		padBottom			= 3,	
		header				= false,
		headerHeight		= 21,
		footer				= false,
		footerHeight		= 21,
		sizerR				= true,
		sizerB				= true,
		sizerBR				= true,
		sizerWidth			= 6,
		sizerHeight			= 6,
		sizerBRHeight		= 6,
		sizerBRWidth		= 6,	
	}		
	-- ~~ Events ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	-- OnAcquire, OnRelease, OnHeightSet, OnWidthSet	
	-- OnSizeChanged, OnDragStop, OnHide, OnShow, OnClose		
	-- ~~ ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~		
	frame:EnableMouse()
	frame:SetMovable(true)
	frame:SetResizable(true)	
	frame:SetScript("OnMouseDown", function(this,button)
		DiesalGUI:OnMouse(this,button)		
	end)
	frame:SetScript("OnSizeChanged", function(this,width,height)		
		local settings 	= self.settings
		settings.width 	= DiesalTools:Round(width)
		settings.height	= DiesalTools:Round(height)
		
		self:FireEvent( "OnSizeChanged", settings.width, settings.height )
	end)
	frame:SetScript("OnHide",function(this)
		self:FireEvent("OnHide")
	end)
	frame:SetScript("OnShow",function(this)
		self:FireEvent("OnShow")
	end)	
	frame:SetToplevel(true)	
	frame.obj = self	
	local titleBar = self:CreateRegion("Button", 'titleBar', frame)
	titleBar:SetPoint("TOPLEFT")
	titleBar:SetPoint("TOPRIGHT")			
	titleBar:EnableMouse()
	titleBar:SetScript("OnMouseDown",function(this,button)
		DiesalGUI:OnMouse(this,button)
		frame:StartMoving()		
	end)
	titleBar:SetScript("OnMouseUp", function(this)		
		frame:StopMovingOrSizing()	
			
		local settings 	= self.settings		
		settings.top 		= DiesalTools:Round(frame:GetTop())
		settings.left	 	= DiesalTools:Round(frame:GetLeft())
				
		self:UpdatePosition()
		self:FireEvent( "OnDragStop", settings.left, settings.top )
	end)	
	local closeButton = self:CreateRegion("Button", 'closeButton', titleBar)	
	closeButton:SetPoint("TOPRIGHT", 0, 0)	
	closeButton:SetScript("OnClick", function(this,button)
		DiesalGUI:OnMouse(this,button)
		PlaySound("gsTitleOptionExit")
		self:FireEvent("OnClose")
		self:Hide()		
	end)		
	local titletext = self:CreateRegion("FontString", 'titletext', titleBar)	
	titletext:SetPoint("TOPLEFT", 4, -5)
	titletext:SetPoint("TOPRIGHT", -20, -5)
	titletext:SetJustifyH("TOP")
	titletext:SetJustifyH("LEFT")			
	
	self:CreateRegion("Frame", 'header', 	frame)	
	self:CreateRegion("Frame", 'content', 	frame)	
	self:CreateRegion("Frame", 'footer', 	frame)	
	
	local sizerBR = self:CreateRegion("Frame", 'sizerBR', frame)	
	sizerBR:SetPoint("BOTTOMRIGHT",frame,"BOTTOMRIGHT",0,0)	
	sizerBR:EnableMouse()
	sizerBR:SetScript("OnMouseDown",function(this,button)
		DiesalGUI:OnMouse(this,button)
		frame:StartSizing("BOTTOMRIGHT")		
	end)
	sizerBR:SetScript("OnMouseUp", function(this)
		frame:StopMovingOrSizing()
	end)		
	local sizerB = self:CreateRegion("Frame", 'sizerB', frame)		
	sizerB:SetPoint("BOTTOMLEFT",frame,"BOTTOMLEFT",0,0)	
	sizerB:EnableMouse()
	sizerB:SetScript("OnMouseDown",function(this,button)
		DiesalGUI:OnMouse(this,button)
		this:GetParent():StartSizing("BOTTOM")		
	end)
	sizerB:SetScript("OnMouseUp", function(this)
		this:GetParent():StopMovingOrSizing()
	end)	
	local sizerR = self:CreateRegion("Frame", 'sizerR', frame)		
	sizerR:EnableMouse()
	sizerR:SetScript("OnMouseDown",function(this,button)
		DiesalGUI:OnMouse(this,button)
		this:GetParent():StartSizing("RIGHT")		
	end)
	sizerR:SetScript("OnMouseUp", function(this)
		this:GetParent():StopMovingOrSizing()
	end)	
	-- ~~ Methods ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	for method, func in pairs(methods) do
		self[method] = func
	end
	-- ~~ ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	DiesalGUI:RegisterObject(self)
	return self	
end	
DiesalGUI:RegisterObjectConstructor(TYPE,Constructor,VERSION)
