local AddOnName, Env = ... local ADDON = Env[1]
local DB = TableExplorerDB
-- ~~| Libraries |~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
local DiesalGUI = LibStub("DiesalGUI-1.0")
local DiesalTools = LibStub("DiesalTools-1.0")
-- ~~| Lua Upvalues |~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
local sub, format, match, lower							= string.sub, string.format, string.match, string.lower
local tsort														= table.sort

-- ~~| WoW Upvalues |~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

-- ~~| Tree |~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
local Type = "TableExplorerTree"
local Version = 2
-- ~~| Tree StyleSheets |~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
local styleSheet = {
	['content-background'] = {		
		type			= 'texture',
		color			= 'ffffff',
		alpha			= .01,
		offset		= 0,	
	},
}
-- ~~| Tree Locals |~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
local function compare(a,b)
	if a[1] == b[1] then
		return a[2] < b[2]
	else
		return  a[1] <  b[1]
	end
end
local function sortType(value)		
	if type(value) == 'table' then return 1 end
	if type(value) == 'function' then return 2 end
	return 3
end
local function sortTable(t)
	local sortedTable = {}			
	for key, value in pairs(t) do			
		sortedTable[#sortedTable + 1] = {sortType(value),lower(tostring(key)), key  }							
	end		
	sort(sortedTable, compare)
	return sortedTable		
end
-- ~~| Tree Methods |~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
local methods = {
	['OnAcquire'] = function(self)				
		-- self:AddStyleSheet(styleSheet)
		-- self:AddStyleSheet(wireFrameSheet)		
		self:ApplySettings()				
		self:Show()		
	end,
	['OnRelease'] = function(self)		
		
	end,	
	['ApplySettings'] = function(self)		
		
	end,	
	['BuildTree'] = function(self,tname,t)
		local settings = self.settings
		settings.endtime = time() + settings.timeout
		self:ReleaseChildren()
		self:FireEvent("OnStatusChange",'') 		
		
		settings.exploredTableName	= tname or settings.exploredTableName
		settings.exploredTable		= t or settings.exploredTable 
		local sortedTable = sortTable(settings.exploredTable)
		for position, k in ipairs(sortedTable) do
			if settings.endtime == time() then				
				self:ReleaseChildren()							
				self:UpdateHeight()
				self:FireEvent("OnStatusChange",'|cffff0000Table Exploration Timed out.') 	
			return end
			local key = k[3] 				
			local branch = DiesalGUI:Create('TableExplorerBranch')
			branch:SetParentObject(self)
			self:AddChild(branch)	
			if position == #sortedTable then branch:SetSettings({last=true}) end			
			branch:BuildBranch(self,key,settings.exploredTable[key],position,1)				
		end			
	end,
	['UpdateHeight'] = function(self)		
		local height = 0
		
		for i=1 , #self.children do 
			height = height + self.children[i].frame:GetHeight()				
		end		
		height = DiesalTools:Round(height)		
		
		if self.settings.height ~= height then
						
			self.settings.height = height
			self:SetHeight(height) 		
			self:FireEvent("OnHeightChange",height) 
		end
	end,
	['CollapseAll'] = function(self)
		for i=1 , #self.children do 
			self.children[i]:Collapse()				
		end		
	end,
	['ExpandAll'] = function(self)
		for i=1 , #self.children do 
			self.children[i]:Expand()				
		end		
	end,	
}
-- ~~| Tree Constructor |~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
local function Constructor()
	local self 		= DiesalGUI:GetObjectBase(Type)	
	local frame 	= self.frame	
	-- ~~ Default Settings ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	self.defaults = {
		exploreDepth 				= 2,				
		depth							= 0,
	}		
	-- ~~ Events ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	-- OnAcquire, OnRelease, OnHeightSet, OnWidthSet	
	-- OnHeightChange
	-- ~~ Construct ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	local content = self:CreateRegion("Frame", 'content', frame)		
	content:SetAllPoints()		
	-- ~~ Methods ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	for method, func in pairs(methods) do
		self[method] = func
	end
	-- ~~ ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	DiesalGUI:RegisterObject(self)
	return self	
end	
DiesalGUI:RegisterObjectConstructor(Type,Constructor,Version)
